import {
  Button,
  Dialog,
  DialogActions,
  DialogBody,
  DialogContent,
  DialogSurface,
  DialogTitle,
} from "@fluentui/react-components";
import React, { PropsWithChildren } from "react";

type AlertContext = (message: string, title?: string) => Promise<void>;

const AlertContextK = React.createContext<AlertContext | null>(null);

export function AlertDialogProvider(p: PropsWithChildren): React.ReactElement {
  const [open, setOpen] = React.useState(false);

  const [title, setTitle] = React.useState<string | undefined>(undefined);
  const [message, setMessage] = React.useState("");

  const cb = React.useRef<null | (() => void)>(null);

  const handleClose = () => {
    setOpen(false);

    if (cb.current !== null) cb.current();
    cb.current = null;
  };

  const hook: AlertContext = (message, title) => {
    setTitle(title);
    setMessage(message);
    setOpen(true);

    return new Promise((res) => {
      cb.current = res;
    });
  };

  return (
    <>
      <AlertContextK.Provider value={hook}>{p.children}</AlertContextK.Provider>

      <Dialog
        open={open}
        onOpenChange={(_event, data) => {
          if (!data.open) setOpen(data.open);
        }}
      >
        <DialogSurface>
          <DialogBody>
            {title && <DialogTitle>{title}</DialogTitle>}
            <DialogContent>{message}</DialogContent>
            <DialogActions>
              <Button onClick={handleClose} autoFocus appearance="secondary">
                Ok
              </Button>
            </DialogActions>
          </DialogBody>
        </DialogSurface>
      </Dialog>
    </>
  );
}

export function useAlert(): AlertContext {
  return React.useContext(AlertContextK)!;
}
